# from typingsql.protocols.connection import ConnectionProtocol


# class Connection(ConnectionProtocol):
#     def __init__(self, connection: ConnectionProtocol):
#         self._connection = connection
